"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolvePathRelativeToConfig = resolvePathRelativeToConfig;
exports.resolveProjectName = resolveProjectName;
exports.resolveSchemaPath = resolveSchemaPath;
exports.getGraphQLProjects = getGraphQLProjects;
exports.getGraphQLSchemaPaths = getGraphQLSchemaPaths;
exports.getGraphQLProjectIncludedFilePaths = getGraphQLProjectIncludedFilePaths;
exports.getGraphQLProjectForSchemaPath = getGraphQLProjectForSchemaPath;
exports.getIncludesExcludesFromConfig = getIncludesExcludesFromConfig;
exports.defaultGraphQLProjectName = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _fs = require("fs");

var _util = require("util");

var _path = require("path");

var _glob = _interopRequireDefault(require("glob"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var defaultGraphQLProjectName = 'GraphQL'; // temporary utility until `graphql-config` supports this function natively
// see: https://github.com/prisma/graphql-config/pull/113

exports.defaultGraphQLProjectName = defaultGraphQLProjectName;

function resolvePathRelativeToConfig(project, relativePath) {
  return (0, _path.resolve)(project.dirpath, relativePath);
}

function resolveProjectName(project) {
  var defaultName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultGraphQLProjectName;
  // eslint-disable-next-line no-console
  console.warn('Deprecation: Use of `resolveProjectName` has been deprecated. Please use `project.name` instead.');
  return project.name || defaultName;
}

function resolveSchemaPath(project) {
  var ignoreMissing = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  // schemaPath is nullable in graphql-config even though it cannot actually be
  // omitted. This function simplifies access to the schema without
  // requiring a type guard.
  if (!project.schema) {
    // this case should never happen with a properly formatted config file.
    // graphql-config currently does not perform any validation so it's possible
    // for a mal-formed schema to be loaded at runtime.
    throw new Error("Missing GraphQL schema for project '".concat(project.name, "'"));
  } // resolve fully qualified schemaPath


  var schemaPath = (0, _path.resolve)(project.dirpath, project.schema);

  if (ignoreMissing) {
    return schemaPath;
  }

  if (!(0, _fs.existsSync)(schemaPath)) {
    var forProject = project.name ? " for project '".concat(project.name, "'") : '';
    throw new Error(["Schema not found".concat(forProject, "."), "Expected to find the schema at '".concat(schemaPath, "' but the path does not exist."), "Check '".concat(project.filepath, "' and verify that schemaPath is configured correctly").concat(forProject, ".")].join(' '));
  }

  return schemaPath;
}

function getGraphQLProjects(config) {
  var _projects$;

  var projects = Object.values((config === null || config === void 0 ? void 0 : config.projects) || {});

  if (projects.length > 1) {
    // multi-project configuration, return an array of projects
    return projects;
  }

  if (projects[0] && (_projects$ = projects[0]) !== null && _projects$ !== void 0 && _projects$.schema) {
    // single project configuration, return an array of the single project
    return projects;
  }

  var defaultProject;

  try {
    defaultProject = config.getDefault();
    return [defaultProject];
  } catch (_unused) {
    // invalid project configuration
    throw new Error("No projects defined in '".concat(config.filepath, "'"));
  }
}

function getGraphQLSchemaPaths(config) {
  return getGraphQLProjects(config).reduce(function (schemas, project) {
    return schemas.concat(resolveSchemaPath(project));
  }, []);
}

function getGraphQLProjectIncludedFilePaths(_x) {
  return _getGraphQLProjectIncludedFilePaths.apply(this, arguments);
}

function _getGraphQLProjectIncludedFilePaths() {
  _getGraphQLProjectIncludedFilePaths = _asyncToGenerator( /*#__PURE__*/_regenerator["default"].mark(function _callee(projectConfig) {
    var _getIncludesExcludesF, _getIncludesExcludesF2, includes, excludes;

    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _getIncludesExcludesF = getIncludesExcludesFromConfig(projectConfig), _getIncludesExcludesF2 = _slicedToArray(_getIncludesExcludesF, 2), includes = _getIncludesExcludesF2[0], excludes = _getIncludesExcludesF2[1];
            _context.next = 3;
            return Promise.all(includes.map(function (include) {
              return (0, _util.promisify)(_glob["default"])(resolvePathRelativeToConfig(projectConfig, include), {
                ignore: excludes.map(function (exclude) {
                  return resolvePathRelativeToConfig(projectConfig, exclude);
                })
              });
            }));

          case 3:
            return _context.abrupt("return", _context.sent.reduce(function (allFilePaths, filePaths) {
              return allFilePaths.concat(filePaths);
            }, []));

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _getGraphQLProjectIncludedFilePaths.apply(this, arguments);
}

function getGraphQLProjectForSchemaPath(config, schemaPath) {
  var project = getGraphQLProjects(config).filter(function (project) {
    return resolveSchemaPath(project, true) === schemaPath;
  }).shift() || config.projects[0];

  if (!project || !project.schema || "".concat(project.dirpath, "/").concat(project.schema) !== schemaPath) {
    throw new Error("No project defined in graphql config for schema '".concat(schemaPath, "'"));
  }

  return project;
}

function getIncludesExcludesFromConfig(projectConfig) {
  if (!projectConfig.include) {
    return [[], []];
  }

  var excludes = projectConfig.exclude || [];
  var includes = typeof projectConfig.include === 'string' ? [projectConfig.include] : projectConfig.include;
  var ignore = typeof excludes === 'string' ? [excludes] : excludes;
  return [includes, ignore];
}